@@ -1,13 +1,12 @@
-import datetime
 import json
 import logging
 import os
-import re
 from collections import Counter
 from collections.abc import Iterable
+from datetime import datetime, timedelta
 from enum import Enum
 from pprint import pformat
-from typing import Dict, Iterator, List, Optional, Tuple, Union
+from typing import Dict, List, Union
 
 import pandas as pd
 import requests
@@ -15,22 +14,19 @@ from cached_property import cached_property
 
 from .exceptions import (
     CityAirException, EmptyDataException, NoAccessException, ServerException,
-    anonymize_request,
-)
+    TransportException, anonymize_request,
+    )
 from .settings import (
     DEFAULT_HOST, DEVICES_PACKETS_URL, DEVICES_URL,
-    FULL_LOGS_URL, LOGS_URL, LOG_CHECKINFO_ADDITIONAL_FILTER_SUFFIX,
-    LOG_CHECKINFO_FILTER_PATTERN, LOG_EXTRACT_PATTERN,
-    LOG_PACKET_ADDITIONAL_FILTER_SUFFIX, LOG_PACKET_FILTER_PATTERN,
-    PACKET_SENDER_IDS, STATIONS_PACKETS_URL, STATIONS_URL,
+    STATIONS_PACKETS_URL, STATIONS_URL,
     TOKEN_VAR_NAME,
-)
+    )
 from .utils import (
     MAIN_DEVICE_PARAMS, MAIN_STATION_PARAMS, RIGHT_PARAMS_NAMES,
     USELESS_COLS, add_progress_bar,
-    parse_checkinfo, prep_df, prep_dicts, timeit, to_date,
-    unpack_cols,
-)
+    prep_df, prep_dicts, timeit, to_date,
+    unpack_cols, is_main_device
+    )
 
 
 class Period(Enum):
@@ -72,12 +68,13 @@ class CityAirRequest:
             try:
                 token = os.environ[TOKEN_VAR_NAME]
             except KeyError:
-                msg = f"Could not find \"{TOKEN_VAR_NAME}\" in environment " \
-                    f"variables"
+                msg = (f"Could not find \"{TOKEN_VAR_NAME}\" in environment "
+                       f"variables")
                 if silent:
                     raise ValueError(msg)
                 token = input(f"{msg}\n please specify you cityair.io token: ")
             self.token = token
+        self.logger = logging.getLogger(__name__)
 
     @cached_property
     def _device_by_serial(self):
@@ -108,7 +105,7 @@ class CityAirRequest:
         return dict(zip(
                 [info['ValueType'] for info in value_types_data],
                 [info['TypeName'] for info in value_types_data]
-        ))
+                ))
 
     @cached_property
     def _device_by_id(self):
@@ -167,9 +164,13 @@ class CityAirRequest:
         -------"""
         body = {"Token": getattr(self, 'token'), **kwargs}
         url = f"{self.host_url}/{method_url}"
-        response = requests.post(url, json=body, timeout=self.timeout,
-                                 verify=self.verify_ssl)
-        logging.debug("post request to url: %s\nbody:%s", url, pformat(body))
+        try:
+            response = requests.post(url, json=body, timeout=self.timeout,
+                                     verify=self.verify_ssl)
+            self.logger.debug("post request to url: %s\n"
+                              "body:%s", url, pformat(body))
+        except requests.exceptions.ConnectionError as e:
+            raise CityAirException(f"Got connection error: {e}") from e
         try:
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
@@ -177,8 +178,7 @@ class CityAirRequest:
         try:
             response_json = response.json()
         except json.JSONDecodeError as e:
-            raise CityAirException(
-                    f"Suddenly got empty json. Couldn't decode it: {e}") from e
+            raise TransportException(response) from e
         if response_json.get('IsError'):
             raise ServerException(response)
         response_data = response_json.get('Result')
@@ -186,9 +186,10 @@ class CityAirRequest:
             if len(response_data[key]) == 0:
                 if not silent:
                     raise EmptyDataException(response=response)
-                logging.warning("There are no %s available.\n"
-                                "url:%s\n"
-                                "filter%s", key, url, anonymize_request(body))
+                self.logger.warning("There are no %s available.\n"
+                                    "url:%s\n"
+                                    "filter%s", key, url,
+                                    anonymize_request(body))
         if len(keys) == 0:
             return response_data
         elif len(keys) == 1:
@@ -224,7 +225,7 @@ class CityAirRequest:
 
         if not include_offline:
             df = df[df['is_online']]
-        df_with_children = df.copy()
+        df_with_children = df.copy(deep=True)
         for children in df['children']:
             for child in children:
                 df_with_children = df_with_children.append(child,
@@ -248,7 +249,10 @@ class CityAirRequest:
                 res.append(single_dict)
             return res
         elif format == 'list':
-            return list(df.index)
+            devices = df.index
+            if not include_children:
+                devices = filter(is_main_device, devices)
+            return list(devices)
         elif format == 'df':
             return df.set_index('serial_number')
         else:
@@ -259,9 +263,9 @@ class CityAirRequest:
     @add_progress_bar
     def get_device_data(self, serial_number: str, start_date=None,
                         finish_date=None, last_packet_id=None,
-                        take_count: int = 500, all_cols=False,
-                        format: str = 'df') \
-            -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
+                        skip_count: int = 0, take_count: int = 500,
+                        all_cols=False, format: str = 'df'
+                        ) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
         """
         Provides data from the selected device
 
@@ -273,6 +277,9 @@ class CityAirRequest:
             dates on which data is being queried
         last_packet_id: int, default None
             if passed, packets will be queried starting from this packet_id
+        skip_count: int, default 0
+            if last_packet_id is passed, number of packets to skip form last
+            packet id
         take_count: int, default 1000
             count of packets which is requested from the server
         all_cols: bool, default False
@@ -298,8 +305,8 @@ class CityAirRequest:
         filter_ = {
                 'Take': take_count,
                 'DeviceId': device_id
-        }
-        if last_packet_id:
+                }
+        if last_packet_id is not None:
             filter_['FilterType'] = 2
             filter_['LastPacketId'] = last_packet_id
         elif start_date:
@@ -307,10 +314,10 @@ class CityAirRequest:
             filter_['TimeBegin'] = to_date(start_date).isoformat()
             filter_['TimeEnd'] = to_date(
                     finish_date).isoformat() if finish_date else \
-                datetime.datetime.now().isoformat()
+                datetime.now().isoformat()
         else:
             filter_['FilterType'] = 3
-            filter_['Skip'] = 0
+            filter_['Skip'] = skip_count
         packets = self._make_request(DEVICES_PACKETS_URL, 'Packets',
                                      Filter=filter_, silent=False)
         df = pd.DataFrame.from_records(packets)
@@ -378,7 +385,7 @@ class CityAirRequest:
                     f"formats are: 'df', 'dict'")
 
     def get_stations(self, format: str = 'list',
-                     include_offline: bool = True
+                     include_offline: bool = True, include_3rd_party=False,
                      ) -> Union[List[str], pd.DataFrame, List[dict]]:
         """
         Provides devices information in various formats
@@ -394,6 +401,8 @@ class CityAirRequest:
             server, other params are ignored
         include_offline: bool, default True
            whether to include offline devices to the output
+        include_3rd_party: bool, default False
+            whether to include 3rd party sources (not having devices)
         -------"""
         locations_data, stations_data, devices_data = self._make_request(
                 STATIONS_URL, "Locations",
@@ -421,7 +430,9 @@ class CityAirRequest:
         df.drop(['devices_auto', 'devices_manual'], axis=1, inplace=True)
         df['location'] = df['location'].apply(
                 lambda id_: locations.get(id_, None) if id_ else None)
-
+        if not include_3rd_party:
+            df = df[df["devices"].apply(lambda serials: any(
+                    filter(lambda serial: serial.startswith("CA"), serials)))]
         if not include_offline:
             df = df[df['is_online']]
         if format == 'dicts':
@@ -440,8 +451,9 @@ class CityAirRequest:
                     f"formats are: 'list', 'df', 'dicts', 'raw'")
 
     @add_progress_bar
-    def get_station_data(self, station_id: int, start_date=None,
-                         finish_date=datetime.datetime.now(),
+    def get_station_data(self, station_id: int,
+                         start_date: Union[str, datetime, None] = None,
+                         finish_date: Union[str, datetime, None] = None,
                          take_count: int = 1000,
                          period: Period = Period.TWENTY_MINS) -> pd.DataFrame:
         """
@@ -459,20 +471,16 @@ class CityAirRequest:
         verbose: bool, default True:
             whether to show progress bar
         -------"""
+        start_date = start_date or (datetime.now() - timedelta(weeks=1))
+        finish_date = finish_date or datetime.utcnow()
         filter_ = {
                 'TakeCount': take_count,
                 'MoId': station_id,
-                'IntervalType': period.value
-        }
-        if start_date:
-            filter_['FilterType'] = 1
-            filter_['BeginTime'] = to_date(start_date).isoformat()
-            filter_['EndTime'] = to_date(
-                    finish_date).isoformat() if finish_date else \
-                datetime.datetime.now().isoformat()
-        else:
-            filter_['FilterType'] = 3
-            filter_['SkipFromLast'] = 0
+                'IntervalType': period.value,
+                'FilterType': 1,
+                'BeginTime': to_date(start_date, format="str"),
+                'EndTime': to_date(finish_date, format="str")
+                }
         packets = self._make_request(STATIONS_PACKETS_URL, 'Packets',
                                      Filter=filter_, silent=False)
         df = pd.DataFrame.from_records(packets)
@@ -508,148 +516,5 @@ class CityAirRequest:
                     location_data.get('name'))
         return locations_data
 
-    def _expand_log_item(self, log_item: dict, extract_re) -> dict:
-        """
-        if log items has longer version it's making request and
-        retrieves full version
-        adds "msg" key to the log_item
-        also extracts packet and adds "packet" key to the log_item
-        """
-        if log_item['IsMessageExt']:
-            full_log_items = self._make_request(FULL_LOGS_URL,
-                                                "LogsItems",
-                                                LogIds=[log_item["LogId"]])
-            log_item.update(msg=full_log_items[0]['Message'])
-        else:
-            log_item.update(msg=log_item['MessageShort'])
-        log_item.update(packet=extract_re.search(log_item['msg']).group(1))
-        return log_item
-
-    def get_logs(self, serial_number: str, type: str = "packet",
-                 start_date=None, finish_date=None, take_count=1000,
-                 app_sender_ids=PACKET_SENDER_IDS, to_include_date=False,
-                 extract_pattern=LOG_EXTRACT_PATTERN, filter_pattern="") \
-            -> Iterator[Union[str, Tuple[datetime.datetime, str]]]:
-        """
-        retrieves log records and extracts records of specific types
-        Parameters
-        ----------
-        serial_number: str
-            serial of the device, also could be any string for backend to
-            filter
-            log messages
-        type: str {'packet', 'checkinfo', 'custom'}, default 'packet'
-            * 'packet' - searching log messages for raw_packets sent by device
-            * 'checkinfo' - searching for check info packets
-            * 'custom' - to auto filtering is performed, extract_pattern and
-              search_pattern args should be provided to perform filtering
-        start_date: datetime or str, default None
-        finish_date: datetime or str, default None
-        take_count: int, default 1000
-            packets count are fetched before filtering
-        app_sender_ids: dict, default [{"AppId": 4, "SenderIds": [23]},
-                                       {"AppId": 2, "SenderIds": [7]}]
-            passed to backend for filtering,
-        to_include_date: bool, default False
-            if to include date to the return
-        extract_pattern: str, default "#([^']*)##"
-            regex for extraction of packet from log message
-        filter_pattern: str, r"#PT#d+" if type='packet'
-                             "#CheckInfo#{[^']*}" if type='checkinfo'
-            regex for filtering log records
-        Returns
-        -------
-            list of log_messages or tuples(datetime, log_message)
-
-        """
-        if type == 'packet':
-            filter_pattern = LOG_PACKET_FILTER_PATTERN
-            serial_number += LOG_PACKET_ADDITIONAL_FILTER_SUFFIX
-        elif type == 'checkinfo':
-            filter_pattern = LOG_CHECKINFO_FILTER_PATTERN
-            serial_number += LOG_CHECKINFO_ADDITIONAL_FILTER_SUFFIX
-        elif type == 'custom':
-            pass
-        else:
-            raise ValueError("type arg should one of the ('packet', "
-                             "'checkinfo', 'custom')")
-        filter_re = re.compile(filter_pattern)
-        extract_re = re.compile(extract_pattern)
-
-        filter_ = {
-                'AppSenderIds': app_sender_ids,
-                'BeginDate': to_date(start_date, format='str'),
-                'EndDate': to_date(finish_date, format='str'),
-                'EventTypeIds': [],
-                'MaxLogItemsCount': take_count,
-                'MessageFilterString': serial_number
-        }
-        log_items = self._make_request(LOGS_URL, "LogsItems", Filter=filter_)
-        filtered_log_items = filter(
-                lambda item: filter_re.search(item['MessageShort']),
-                log_items)
-        full_items = map(lambda item: self._expand_log_item(item, extract_re),
-                         filtered_log_items)
-
-        for log_item in full_items:
-            if to_include_date:
-                yield to_date(log_item['Time']), log_item['packet']
-            else:
-                yield log_item['packet']
-
-    def get_last_log(self, serial_number: str, type: str = "packet") -> str:
-        """
-        retrieves log records and extracts records of specific types
-        Parameters
-        ----------
-        serial_number: str
-        type: str {'packet', 'checkinfo'}, default 'packet'
-            * 'packet' - searching log messages for raw_packets sent by device
-            * 'checkinfo' - searching for check info packets
-        Returns
-        -------
-            last log message
-
-        """
-        if type not in ('packet', 'checkinfo'):
-            raise ValueError(
-                    "type arg shoud be either 'packet' or 'checkinfo'")
-        msgs = self.get_logs(serial_number, type=type, take_count=1)
-        return next(msgs)
-
-    def get_last_rawpacket(self, serial_number: str) -> Optional[str]:
-        """
-        retrieves filtered log records and extracts packet sent by device
-        Parameters
-        ----------
-        serial_number: str
-        Returns
-        -------
-            last raw packet
-
-        """
-        msgs = self.get_logs(serial_number, type="packet", take_count=1)
-        try:
-            return next(msgs)
-        except StopIteration:
-            return None
-
-    def get_last_checkinfo(self, serial_number: str) -> Optional[List[dict]]:
-        """
-        retrieves filtered log records and extracts checkinfo sent by device
-        Parameters
-        ----------
-        serial_number: str
-        Returns
-        -------
-            last checkinfo
-
-        """
-        msg = (self.get_logs(serial_number, type="checkinfo", take_count=1))
-        try:
-            return parse_checkinfo(next(msg))
-        except StopIteration:
-            return None
-
 
 CAR = CityAirRequest
