@@ -14,6 +14,18 @@ import pytz
 from cityair_api.settings import CHECKINFO_PARSE_PATTERN
 from .exceptions import EmptyDataException
 
+logger = logging.getLogger(__name__)
+
+FATHER_PREFIXES = ["CA", "ROOFTOP", "LAB"]
+
+
+def is_main_device(serial):
+    for prefix in FATHER_PREFIXES:
+        if serial.startswith(prefix):
+            return True
+    return False
+
+
 RIGHT_PARAMS_NAMES = {
         'FlagPs220': '220',
         'RecvDate': 'receive_date', 'Ps220': '220',
@@ -54,7 +66,7 @@ RIGHT_PARAMS_NAMES = {
         'NameRu': 'name_ru',
         'PacketId': 'packet_id',
         "Humidity_": "RH"
-}
+        }
 USELESS_COLS = ['220', 'BatLow', 'receive_date', 'GeoInfo', 'Geo', 'Date',
                 'SendDate', 'ResetMoData', 'description', 'coordinates',
                 'rssi', 'FlagBatLowHasFailed', 'FlagPs220HasFailed',
@@ -65,7 +77,7 @@ USELESS_COLS = ['220', 'BatLow', 'receive_date', 'GeoInfo', 'Geo', 'Date',
                 'BounceWest', 'CountryId', 'BounceNorth', 'BounceSouth',
                 'BounceEast', 'BounceWest', 'GmtHour', 'LocationUrl',
                 'DistributionSummary', 'SortRank', 'PacketId', 'packet_id',
-                'Pcf', 'Scf']
+                "DoS", "DDW", 'Pcf', 'Scf']
 
 MAIN_DEVICE_PARAMS = ['serial_number', 'name', 'software', 'stations',
                       'children', 'check_infos']
@@ -86,11 +98,8 @@ def add_progress_bar(method):
         start_date = to_date(kwargs.get('start_date'))
         if not start_date:
             return method(*args, **kwargs)
-        finish_date = to_date(kwargs.get('finish_date',
-                                         datetime.datetime.utcnow().replace(
-                                                 tzinfo=pytz.utc)))
-        start_date.replace(tzinfo=None)
-        finish_date.replace(tzinfo=None)
+        finish_date = to_date(
+                kwargs.get('finish_date', datetime.datetime.utcnow()))
         kwargs.update(take_count=kwargs.get('take_count', default_take_count))
         bar = progressbar.ProgressBar(max_value=(finish_date - start_date)
                                       .total_seconds() / progress_scaler,
@@ -102,7 +111,7 @@ def add_progress_bar(method):
                                               progressbar.Timer(),
                                               progressbar.Bar(),
                                               progressbar.ETA()
-                                      ],
+                                              ],
                                       max_error=False
                                       )
         if kwargs.get('format', 'df') == 'df':
@@ -123,14 +132,15 @@ def add_progress_bar(method):
                 start_date += datetime.timedelta(days=2)
             kwargs.update(
                     start_date=start_date + datetime.timedelta(seconds=30))
-            bar.update(bar.max_value - (
-                    finish_date - start_date).total_seconds() /
-                       progress_scaler)
-        size = len(res) if isinstance(res, pd.DataFrame) \
-            else max(map(len, res.values()))
-        logging.info(f'finished acquiring {args[1]} data of size {size}')
+            fetched_seconds = (finish_date.replace(tzinfo=pytz.utc)
+                               - start_date.replace(
+                            tzinfo=pytz.utc)).total_seconds()
+            bar.update(bar.max_value - (fetched_seconds / progress_scaler))
+        size = len(res) if isinstance(res, pd.DataFrame) else max(
+                map(len, res.values()))
+        logger.info(f'finished acquiring {args[1]} data of size {size}')
         if size == 0:
-            raise EmptyDataException
+            raise EmptyDataException(request_args=kwargs)
         return res
 
     return progressed
@@ -180,7 +190,7 @@ def parse_checkinfo(msg: str) -> List[dict]:
     return parsed_checkinfo
 
 
-def to_date(date: Optional[Union[datetime.datetime, str]],
+def to_date(date: Union[datetime.datetime, str, None],
             format: str = 'date') -> Optional[Union[str, datetime.datetime]]:
     """
 
@@ -194,7 +204,7 @@ def to_date(date: Optional[Union[datetime.datetime, str]],
     if isinstance(date, datetime.datetime):
         pass
     elif isinstance(date, str):
-        return pd.to_datetime(date, dayfirst=True, utc=True)
+        date = pd.to_datetime(date, dayfirst=True)
     else:
         raise ValueError(f"date should be 'str' or 'datetime', "
                          f"got {type(date)} instead")
@@ -202,7 +212,7 @@ def to_date(date: Optional[Union[datetime.datetime, str]],
     if format == 'str':
         return date.isoformat()
     elif format == 'date':
-        return date
+        return date.replace(tzinfo=None)
     else:
         raise ValueError(f"format arg should be one of the 'str', 'date', "
                          f"got {format} instead")
@@ -238,6 +248,7 @@ def prep_df(df: pd.DataFrame, right_param_names: dict = RIGHT_PARAMS_NAMES,
         res.set_index(index_col, inplace=True)
     res.rename(right_param_names, axis=1, inplace=True)
     res = res.drop(cols_to_drop, axis=1, errors='ignore')
+    res = res.sort_index()
     return res
 
 
@@ -253,7 +264,7 @@ def timeit(method):
         result = method(*args, **kwargs)
         te = time.time()
 
-        logging.debug(
+        logger.debug(
                 f"{te - ts:.2f} seconds took to {method.__name__} of size"
                 f"{sys.getsizeof(result) / 1000: .2f} KB\nargs were"
                 f" {', '.join(map(str, args[1:]))}\nkwargs were: {kwargs}")
